#include "RawPacket.h"


RawPacket::RawPacket (unsigned int protocol, int buf_size)
{
#if defined(__WIN32)
  this->protocol = protocol;
  output = new unsigned char[buf_size];
#else
  this->buf_size = buf_size;
  this->protocol = protocol;
  sock = socket(PF_PACKET, SOCK_RAW, htons(protocol));
  if (sock < 0){
    perror("socket");
    return;
  }
  output = new unsigned char[buf_size];
  input = new unsigned char[buf_size];
#endif
}

RawPacket::~RawPacket ()
{
  Close();
#if defined(__WIN32)
  delete [] output;
#else
  delete [] input;
  delete [] output;
#endif
}

int
RawPacket::Err(char *msg, int errcode)
{
  fprintf(stderr, "%s\n", msg);
  return errcode;
}

void
RawPacket::Msg(char *msg)
{
  printf("%s\n", msg);
}

void
RawPacket::dump(int size, unsigned char *p)
{
  int i;
  for (i=0;i<size;i++){
    if (i%16==0) printf("\n%.8X: ", i);
      printf("%.2X ", p[i]);
      if (i%16==7) printf("- ");
  }
  printf("\n");
}

int 
RawPacket::Read()
{
#if defined(__WIN32)
  int n = 0;
  struct pcap_pkthdr pkthdr;
  (const u_char*)input = pcap_next(iface, &pkthdr);
  if (input==NULL) return 0;
  input_size = pkthdr.len;
  return pkthdr.len;
#else
  int n = 0;
  socklen_t ssize = sizeof(dev);
  n = recvfrom(sock, input, buf_size, 0, (sockaddr *)&dev, &ssize);
  input_size = n;
  if (n<0){
    perror("recvfrom");
    return -1;
  }
  return n;
#endif
}

int 
RawPacket::Write(int size)
{
#if defined(__WIN32)
  int n = pcap_sendpacket(iface, (u_char*)output, size);
  return n;
#else
  int n;
  n = sendto(sock, output, size, 0, (sockaddr *)&dev, sizeof(dev));
  if (n<0) {perror("sendto"); return -1;}
  return n;
#endif
}

int 
RawPacket::Open(const char *ifname)
{
#if defined(__WIN32)
  iface = pcap_open_live(ifname, 65536, 0, 1000, errbuf);
  if (iface == NULL) {
    Err(errbuf);
    return -1;
  }
  strcpy(this->ifname, ifname);
  if (protocol) {
    struct bpf_program fcode;
    char buf[256];
    unsigned char * mac = GetMacAddr();
    sprintf(buf,"ether[12:2] = 0x%.4x and ether dst %02x:%02x:%02x:%02x:%02x:%02x" ,protocol, mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
    if (pcap_compile(iface, &fcode, buf, 1, 0)==-1){
      Err(pcap_geterr(iface));
      exit(0);
    }
    if (pcap_setfilter(iface, &fcode) < 0){
      Err(pcap_geterr(iface));
      exit(-1);
    }
  }
  return 0;
#else
  int n;
  strncpy(req.ifr_name, ifname, IFNAMSIZ);
  n = ioctl(sock, SIOGIFINDEX, &req);
  if (n==-1){perror("RawPacket::SetDevice [ioctl]"); return -1;}
  ifindex = req.ifr_ifindex;
  dev.sll_family = AF_PACKET;
  dev.sll_protocol = protocol;
  dev.sll_ifindex = req.ifr_ifindex;
  strncpy(this->ifname, ifname, IFNAMSIZ);
  return 0;
#endif
}

int
RawPacket::Close()
{
#if defined(__WIN32)
  pcap_freecode(&fcode);
  pcap_close(iface);
#else
  close(sock);
#endif
}

unsigned char * 
RawPacket::GetMacAddr()
{
#if defined(__WIN32)
  static unsigned char mac[6];
  LPADAPTER ifr;
  PPACKET_OID_DATA oid;
  int v;
  ifr = PacketOpenAdapter((char *)ifname);
  if (ifr == NULL) {
    Err("Cannot get Mac Address.");
    exit(-1);
  }
  (char *)oid = new char[sizeof(PACKET_OID_DATA)+6];
  oid->Oid = OID_802_3_CURRENT_ADDRESS;
  oid->Length = 6;
  PacketRequest(ifr, FALSE, oid);
  memcpy(mac, oid->Data, 6);
  delete oid;
  return mac;
#else
  int n;
  static unsigned char mac[IFHWADDRLEN];
  n = ioctl(sock, SIOCGIFHWADDR, &req);
  if (n == -1){perror("RawPacket::GetMacAddr [ioctl]"); return NULL;}
  memcpy(mac, req.ifr_hwaddr.sa_data, IFHWADDRLEN);
  return mac;
#endif
}

unsigned int
RawPacket::GetIPs()
{
#if defined(__WIN32)
  pcap_if_t *alldevs;
  pcap_if_t *d;
  pcap_findalldevs(&alldevs, errbuf);
  for(d = alldevs; d; d=d->next){
    if (strcmp(d->name, ifname)==0){
      struct pcap_addr * a;
      for(a = d->addresses;a;a=a->next){
	ip = ((struct sockaddr_in *)a->addr)->sin_addr.s_addr;
	mask = ((struct sockaddr_in *)a->netmask)->sin_addr.s_addr;
	board = ((struct sockaddr_in *)a->broadaddr)->sin_addr.s_addr;
      }
      break;
    }
  }
  return 0;
#else
  int n;
  n = ioctl(sock, SIOCGIFADDR, &req);
  if (n == -1){perror("ioctl"); return 0;}
  ip = (*(struct sockaddr_in *)&req.ifr_addr).sin_addr.s_addr;
  n = ioctl(sock, SIOCGIFNETMASK, &req);
  if (n == -1){perror("RawPacket::GetMask [ioctl]"); return 0;}
  mask = (*(struct sockaddr_in *)&req.ifr_netmask).sin_addr.s_addr;
  n = ioctl(sock, SIOCGIFBRDADDR, &req);
  if (n == -1){perror("RawPacket::GetBroad [ioctl]"); return 0;}
  board = (*(struct sockaddr_in *)&req.ifr_broadaddr).sin_addr.s_addr;
  return 0;
#endif
}

